@@ -65,6 +65,7 @@ import ssl
 import sys
 import threading
 
+from h2.config import H2Configuration
 from h2.connection import H2Connection
 from h2.events import (
     DataReceived, RequestReceived, WindowUpdated, StreamEnded, StreamReset
@@ -86,8 +87,10 @@ APPLICATION = None
 
 class H2Protocol(asyncio.Protocol):
     def __init__(self):
+        config = H2Configuration(client_side=False, header_encoding='utf-8')
+
         # Our server-side state machine.
-        self.conn = H2Connection(client_side=False)
+        self.conn = H2Connection(config=config)
 
         # The backing transport.
         self.transport = None
@@ -181,7 +184,7 @@ class H2Protocol(asyncio.Protocol):
             # This is specific to a single stream.
             if event.stream_id in self._flow_controlled_data:
                 self._stream_data.put_nowait(
-                    self._flow_controlled_data.pop(stream_id)
+                    self._flow_controlled_data.pop(event.stream_id)
                 )
         else:
             # This event is specific to the connection. Free up *all* the
