@@ -10,6 +10,7 @@ import pytest
 from hypothesis import given
 from hypothesis.strategies import integers
 
+import h2.config
 import h2.connection
 import h2.errors
 import h2.events
@@ -27,6 +28,7 @@ class TestFlowControl(object):
         (':scheme', 'https'),
         (':method', 'GET'),
     ]
+    server_config = h2.config.H2Configuration(client_side=False)
 
     DEFAULT_FLOW_WINDOW = 65535
 
@@ -52,12 +54,29 @@ class TestFlowControl(object):
         remaining_length = self.DEFAULT_FLOW_WINDOW - len(b'some data')
         assert (c.local_flow_control_window(1) == remaining_length)
 
+    @pytest.mark.parametrize("pad_length", [5, 0])
+    def test_flow_control_decreases_with_sent_data_with_padding(self,
+                                                                pad_length):
+        """
+        When padded data is sent on a stream, the flow control window drops
+        by the length of the padding plus 1 for the 1-byte padding length
+        field.
+        """
+        c = h2.connection.H2Connection()
+        c.send_headers(1, self.example_request_headers)
+
+        c.send_data(1, b'some data', pad_length=pad_length)
+        remaining_length = (
+            self.DEFAULT_FLOW_WINDOW - len(b'some data') - pad_length - 1
+        )
+        assert c.local_flow_control_window(1) == remaining_length
+
     def test_flow_control_decreases_with_received_data(self, frame_factory):
         """
         When data is received on a stream, the remote flow control window
         should drop.
         """
-        c = h2.connection.H2Connection(client_side=False)
+        c = h2.connection.H2Connection(config=self.server_config)
         c.receive_data(frame_factory.preamble())
         f1 = frame_factory.build_headers_frame(self.example_request_headers)
         f2 = frame_factory.build_data_frame(b'some data')
@@ -70,9 +89,9 @@ class TestFlowControl(object):
     def test_flow_control_decreases_with_padded_data(self, frame_factory):
         """
         When padded data is received on a stream, the remote flow control
-        window should drop by an amount that includes the padding.
+        window drops by an amount that includes the padding.
         """
-        c = h2.connection.H2Connection(client_side=False)
+        c = h2.connection.H2Connection(config=self.server_config)
         c.receive_data(frame_factory.preamble())
         f1 = frame_factory.build_headers_frame(self.example_request_headers)
         f2 = frame_factory.build_data_frame(b'some data', padding_len=10)
@@ -102,7 +121,7 @@ class TestFlowControl(object):
         The remote flow control window is limited by the flow control of the
         connection.
         """
-        c = h2.connection.H2Connection(client_side=False)
+        c = h2.connection.H2Connection(config=self.server_config)
         c.receive_data(frame_factory.preamble())
         f1 = frame_factory.build_headers_frame(self.example_request_headers)
         f2 = frame_factory.build_data_frame(b'some data')
@@ -182,7 +201,7 @@ class TestFlowControl(object):
         assert c.local_flow_control_window(1) == 65535
 
         f = frame_factory.build_settings_frame(
-            settings={h2.settings.INITIAL_WINDOW_SIZE: 1280}
+            settings={h2.settings.SettingCodes.INITIAL_WINDOW_SIZE: 1280}
         )
         c.receive_data(f.serialize())
 
@@ -206,7 +225,7 @@ class TestFlowControl(object):
         assert c.local_flow_control_window(1) == 65535
 
         f = frame_factory.build_settings_frame(
-            settings={h2.settings.INITIAL_WINDOW_SIZE: 128000}
+            settings={h2.settings.SettingCodes.INITIAL_WINDOW_SIZE: 128000}
         )
         c.receive_data(f.serialize())
 
@@ -224,7 +243,7 @@ class TestFlowControl(object):
         assert c.local_flow_control_window(1) == 65535
 
         f = frame_factory.build_settings_frame(
-            settings={h2.settings.INITIAL_WINDOW_SIZE: 128000}
+            settings={h2.settings.SettingCodes.INITIAL_WINDOW_SIZE: 128000}
         )
         c.receive_data(f.serialize())
 
@@ -238,7 +257,7 @@ class TestFlowControl(object):
         c = h2.connection.H2Connection()
 
         f = frame_factory.build_settings_frame(
-            settings={h2.settings.INITIAL_WINDOW_SIZE: 128000}
+            settings={h2.settings.SettingCodes.INITIAL_WINDOW_SIZE: 128000}
         )
         c.receive_data(f.serialize())
 
@@ -256,7 +275,7 @@ class TestFlowControl(object):
         WindowUpdate frames received without streams fire an appropriate
         WindowUpdated event.
         """
-        c = h2.connection.H2Connection(client_side=False)
+        c = h2.connection.H2Connection(config=self.server_config)
         c.receive_data(frame_factory.preamble())
 
         f = frame_factory.build_window_update_frame(
@@ -277,7 +296,7 @@ class TestFlowControl(object):
         WindowUpdate frames received with streams fire an appropriate
         WindowUpdated event.
         """
-        c = h2.connection.H2Connection(client_side=False)
+        c = h2.connection.H2Connection(config=self.server_config)
         c.receive_data(frame_factory.preamble())
 
         f1 = frame_factory.build_headers_frame(self.example_request_headers)
@@ -338,12 +357,12 @@ class TestFlowControl(object):
         The user can set a low flow control window, which leads to connection
         teardown if violated.
         """
-        c = h2.connection.H2Connection(client_side=False)
+        c = h2.connection.H2Connection(config=self.server_config)
         c.receive_data(frame_factory.preamble())
 
         # Change the flow control window to 80 bytes.
         c.update_settings(
-            {h2.settings.INITIAL_WINDOW_SIZE: 80}
+            {h2.settings.SettingCodes.INITIAL_WINDOW_SIZE: 80}
         )
         f = frame_factory.build_settings_frame({}, ack=True)
         c.receive_data(f.serialize())
@@ -376,7 +395,7 @@ class TestFlowControl(object):
 
         assert c.remote_flow_control_window(1) == 65535
 
-        c.update_settings({h2.settings.INITIAL_WINDOW_SIZE: 1280})
+        c.update_settings({h2.settings.SettingCodes.INITIAL_WINDOW_SIZE: 1280})
 
         f = frame_factory.build_settings_frame({}, ack=True)
         c.receive_data(f.serialize())
@@ -396,7 +415,9 @@ class TestFlowControl(object):
 
         assert c.remote_flow_control_window(1) == 65535
 
-        c.update_settings({h2.settings.INITIAL_WINDOW_SIZE: 128000})
+        c.update_settings(
+            {h2.settings.SettingCodes.INITIAL_WINDOW_SIZE: 128000}
+        )
         f = frame_factory.build_settings_frame({}, ack=True)
         c.receive_data(f.serialize())
 
@@ -409,7 +430,9 @@ class TestFlowControl(object):
         """
         c = h2.connection.H2Connection()
 
-        c.update_settings({h2.settings.INITIAL_WINDOW_SIZE: 128000})
+        c.update_settings(
+            {h2.settings.SettingCodes.INITIAL_WINDOW_SIZE: 128000}
+        )
         f = frame_factory.build_settings_frame({}, ack=True)
         c.receive_data(f.serialize())
 
@@ -505,11 +528,17 @@ class TestFlowControl(object):
             stream_id=1, increment=increment
         )
 
-        with pytest.raises(h2.exceptions.FlowControlError):
-            c.receive_data(f.serialize())
+        events = c.receive_data(f.serialize())
+        assert len(events) == 1
 
-        expected_frame = frame_factory.build_goaway_frame(
-            last_stream_id=0,
+        event = events[0]
+        assert isinstance(event, h2.events.StreamReset)
+        assert event.stream_id == 1
+        assert event.error_code == h2.errors.ErrorCodes.FLOW_CONTROL_ERROR
+        assert not event.remote_reset
+
+        expected_frame = frame_factory.build_rst_stream_frame(
+            stream_id=1,
             error_code=h2.errors.ErrorCodes.FLOW_CONTROL_ERROR,
         )
         assert c.data_to_send() == expected_frame.serialize()
@@ -533,7 +562,8 @@ class TestFlowControl(object):
         # Receive an increment to the initial window size.
         f = frame_factory.build_settings_frame(
             settings={
-                h2.settings.INITIAL_WINDOW_SIZE: self.DEFAULT_FLOW_WINDOW + 1
+                h2.settings.SettingCodes.INITIAL_WINDOW_SIZE:
+                    self.DEFAULT_FLOW_WINDOW + 1
             }
         )
         c.clear_outbound_data_buffer()
@@ -569,7 +599,8 @@ class TestFlowControl(object):
         # Receive an increment to the initial window size.
         f = frame_factory.build_settings_frame(
             settings={
-                h2.settings.INITIAL_WINDOW_SIZE: self.DEFAULT_FLOW_WINDOW + 1
+                h2.settings.SettingCodes.INITIAL_WINDOW_SIZE:
+                    self.DEFAULT_FLOW_WINDOW + 1
             }
         )
         c.clear_outbound_data_buffer()
@@ -618,6 +649,7 @@ class TestAutomaticFlowControl(object):
         (':scheme', 'https'),
         (':method', 'GET'),
     ]
+    server_config = h2.config.H2Configuration(client_side=False)
 
     DEFAULT_FLOW_WINDOW = 65535
 
@@ -626,12 +658,12 @@ class TestAutomaticFlowControl(object):
         Setup a server-side H2Connection and send a headers frame, and then
         clear the outbound data buffer. Also increase the maximum frame size.
         """
-        c = h2.connection.H2Connection(client_side=False)
+        c = h2.connection.H2Connection(config=self.server_config)
         c.initiate_connection()
         c.receive_data(frame_factory.preamble())
 
         c.update_settings(
-            {h2.settings.MAX_FRAME_SIZE: self.DEFAULT_FLOW_WINDOW}
+            {h2.settings.SettingCodes.MAX_FRAME_SIZE: self.DEFAULT_FLOW_WINDOW}
         )
         settings_frame = frame_factory.build_settings_frame(
             settings={}, ack=True
